﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;

namespace HeuristicLab.Algorithms.Bandits.Models {
  public class GaussianMixtureModel : IModel {
    private readonly double[] componentMeans;
    private readonly double[] componentVars;
    private readonly double[] componentProbs;

    private int numComponents;

    public GaussianMixtureModel(int nComponents = 5) {
      this.numComponents = nComponents;
      this.componentProbs = new double[nComponents];
      this.componentMeans = new double[nComponents];
      this.componentVars = new double[nComponents];
    }


    public double SampleExpectedReward(Random random) {
      var k = Enumerable.Range(0, numComponents).SampleProportional(random, componentProbs).First();
      return alglib.invnormaldistribution(random.NextDouble()) * Math.Sqrt(componentVars[k]) + componentMeans[k];
    }

    public void Update(double reward) {
      // see http://www.cs.toronto.edu/~mackay/itprnn/ps/302.320.pdf Algorithm 22.2 soft k-means
      throw new NotImplementedException();
    }

    public void Disable() {
      Array.Clear(componentMeans, 0, numComponents);
      for (int i = 0; i < numComponents; i++)
        componentVars[i] = 0.0;
    }

    public object Clone() {
      return new GaussianMixtureModel(numComponents);
    }

    public void Reset() {
      Array.Clear(componentMeans, 0, numComponents);
      Array.Clear(componentVars, 0, numComponents);
      Array.Clear(componentProbs, 0, numComponents);
    }

    public void PrintStats() {
      throw new NotImplementedException();
    }
  }
}
